@@ -3,10 +3,9 @@
 #
 #    This file is part of GnuPG.pm.
 #
-#    Author: Francis J. Lacoste <francis.lacoste@Contre.COM>
+#    Author: Francis J. Lacoste <francis.lacoste@iNsu.COM>
 #
-#    Copyright (C) 2000 iNsu Innovations Inc.
-#    Copyright (C) 2001 Francis J. Lacoste
+#    Copyright (C) 1999 Francis J. Lacoste, iNsu Innovations
 #
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -37,43 +36,46 @@ BEGIN {
     @EXPORT = qw();
 
     %EXPORT_TAGS = (
-            algo   => [ qw(    DSA_ELGAMAL DSA ELGAMAL_ENCRYPT
-                    ELGAMAL
-                    ) ],
-            trust  => [ qw(    TRUST_UNDEFINED    TRUST_NEVER
-                    TRUST_MARGINAL    TRUST_FULLY
-                    TRUST_ULTIMATE ) ],
-           );
+		    algo   => [ qw(	DSA_ELGAMAL DSA ELGAMAL_ENCRYPT
+					ELGAMAL
+				    ) ],
+		    trust  => [ qw(	TRUST_UNDEFINED	TRUST_NEVER
+					TRUST_MARGINAL	TRUST_FULLY
+					TRUST_ULTIMATE ) ],
+		   );
 
     Exporter::export_ok_tags( qw( algo trust ) );
 
-    $VERSION = '0.10_1';
+    $VERSION = '0.02';
 }
 
-use constant DSA_ELGAMAL    => 1;
-use constant DSA        => 2;
-use constant ELGAMAL_ENCRYPT    => 3;
-use constant ELGAMAL        => 4;
+use constant DSA_ELGAMAL	=> 1;
+use constant DSA		=> 2;
+use constant ELGAMAL_ENCRYPT	=> 3;
+use constant ELGAMAL		=> 4;
 
-use constant TRUST_UNDEFINED    => -1;
-use constant TRUST_NEVER    => 0;
-use constant TRUST_MARGINAL    => 1;
-use constant TRUST_FULLY    => 2;
-use constant TRUST_ULTIMATE    => 3;
+use constant TRUST_UNDEFINED	=> -1;
+use constant TRUST_NEVER	=> 0;
+use constant TRUST_MARGINAL	=> 1;
+use constant TRUST_FULLY	=> 2;
+use constant TRUST_ULTIMATE	=> 3;
 
 use Carp;
-use POSIX qw();
 use Symbol;
+
+use IO::Select;
+
 use Fcntl;
 
+
 sub parse_trust {
     for (shift) {
-    /ULTIMATE/  && do { return TRUST_ULTIMATE;  };
-    /FULLY/        && do { return TRUST_FULLY;        };
-    /MARGINAL/  && do { return TRUST_MARGINAL;  };
-    /NEVER/        && do { return TRUST_NEVER;        };
-    # Default
-    return TRUST_UNDEFINED;
+	/ULTIMATE/  && do { return TRUST_ULTIMATE;  };
+	/FULLY/	    && do { return TRUST_FULLY;	    };
+	/MARGINAL/  && do { return TRUST_MARGINAL;  };
+	/NEVER/	    && do { return TRUST_NEVER;	    };
+	# Default
+	return TRUST_UNDEFINED;
     }
 }
 
@@ -102,7 +104,7 @@ sub cmdline($) {
     # Default options
     push @$args, "--no-tty" unless $self->{trace};
     push @$args, "--no-greeting", "--status-fd", fileno $self->{status_fd},
-         "--run-as-shm-coprocess", "0";
+		 "--run-as-shm-coprocess", "0";
 
     # Check for homedir and options file
     push @$args, "--homedir", $self->{homedir} if $self->{homedir};
@@ -122,22 +124,18 @@ sub cmdline($) {
 sub end_gnupg($) {
     my $self = shift;
 
-    print STDERR "GnuPG: closing status fd " . fileno ($self->{status_fd})
-      . "\n"
-    if $self->{trace};
-
     close $self->{status_fd}
       or croak "error while closing pipe: $!\n";
 
     waitpid $self->{gnupg_pid}, 0
       or croak "error while waiting for gpg: $!\n";
 
-    for ( qw(protocol gnupg_pid shmid shm_size shm_lock_size
-         command options args status_fd input output
-         next_status ) )
-    {
-    delete $self->{$_};
-    }
+    delete $self->{$_} for ( qw(protocol gnupg_pid
+				shmid shm_size shm_lock_size
+				command options args
+				status_fd input output
+				next_status
+			    ) );
 
 }
 
@@ -146,12 +144,12 @@ sub abort_gnupg($$) {
 
     # Signal our child that it is the end
     if ($self->{gnupg_pid} && kill 0 => $self->{gnupg_pid} ) {
-        kill INT => $self->{gnupg_pid};
+	kill INT => $self->{gnupg_pid};
     }
 
     $self->end_gnupg;
 
-    confess ( $msg );
+    croak ( $msg );
 }
 
 # Used to push back status information
@@ -163,27 +161,28 @@ sub next_status($$$) {
 
 sub read_from_status($) {
     my $self = shift;
+
     # Check if a status was pushed back
     if ( $self->{next_status} ) {
-        my $status = $self->{next_status};
-        $self->{next_status} = undef;
-        return @$status;
+	my $status = $self->{next_status};
+	$self->{next_status} = undef;
+	return @$status;
     }
 
-    print STDERR "GnuPG: reading from status fd " . fileno ($self->{status_fd}) . "\n" if $self->{trace};
-
+    print STDERR "GnuPG: reading from status fd\n"
+      if $self->{trace};
     my $fd = $self->{status_fd};
-    local $/ = "\n"; # Just to be sure
     my $line = <$fd>;
     unless ($line) {
-        print STDERR "GnuPG: got from status fd: EOF" if $self->{trace};
-        return ();
+	print STDERR "GnuPG: got from status fd: EOF" if $self->{trace};
+	return ();
     }
-
-    print STDERR "GnuPG: got from status fd: $line" if $self->{trace};
+    print STDERR "GnuPG: got from status fd: $line"
+      if $self->{trace};
 
     my ( $cmd,$arg ) = $line =~ /\[GNUPG:\] (\w+) ?(.+)?$/;
-    $self->abort_gnupg( "error communicating with gnupg: bad status line: $line\n" ) unless $cmd;
+    $self->abort_gnupg( "error communicating with gnupg: bad status line: $line\n" )
+      unless $cmd;
     return wantarray ? ( $cmd, $arg ) : $cmd;
 }
 
@@ -200,105 +199,90 @@ sub run_gnupg($) {
 
     # Keep pipe open after close
     fcntl( $fd, F_SETFD, 0 )
-    or croak "error removing close on exec flag: $!\n" ;
+	or croak "error removing close on exec flag: $!\n" ;
     fcntl( $wfd, F_SETFD, 0 )
-    or croak "error removing close on exec flag: $!\n" ;
+	or croak "error removing close on exec flag: $!\n" ;
 
     my $pid = fork;
     croak( "error forking: $!" ) unless defined $pid;
     if ( $pid ) {
-    # Parent
-    close $wfd;
+	# Parent
+	$self->{status_fd} = $fd;
+	$self->{gnupg_pid} = $pid;
 
-    $self->{status_fd} = $fd;
-    $self->{gnupg_pid} = $pid;
+	my ($cmd, $arg ) = $self->read_from_status;
 
-    my ($cmd, $arg ) = $self->read_from_status;
+	$self->abort_gnupg( "wrong response from gnupg (expected SHM_INFO): $cmd\n")
+	  unless ( $cmd eq "SHM_INFO" );
 
-    $self->abort_gnupg( "wrong response from gnupg (expected SHM_INFO): $cmd\n")
-      unless ( $cmd eq "SHM_INFO" );
+	my ( $proto, $gpid, $shmid, $sz, $lz ) =
+	  $arg =~ /pv=(\d+) pid=(\d+) shmid=(\d+) sz=(\d+) lz=(\d+)/;
 
-    my ( $proto, $gpid, $shmid, $sz, $lz ) =
-      $arg =~ /pv=(\d+) pid=(\d+) shmid=(\d+) sz=(\d+) lz=(\d+)/;
+	$self->abort_gnupg( "unsupported protocol version: $proto\n" )
+	  unless $proto == 1;
 
-    $self->abort_gnupg( "unsupported protocol version: $proto\n" )
-      unless $proto == 1;
-
-    $self->{protocol}        = $proto;
-    $self->{shmid}            = $shmid;
-    $self->{shm_size}        = $sz;
-    $self->{shm_lock_size}  = $lz;
+	$self->{protocol}		= $proto;
+	$self->{shmid}			= $shmid;
+	$self->{shm_size}		= $sz;
+	$self->{shm_lock_size}  = $lz;
     } else {
-    # Child
-    $self->{status_fd} = $wfd;
+	# Child
+	$self->{status_fd} = $wfd;
 
-    my $cmdline = $self->cmdline;
-    unless ( $self->{trace} ) {
-        open (STDERR, "> /dev/null" )
-           or die "can't redirect stderr to /dev/null: $!\n";
-    }
+	my $cmdline = $self->cmdline;
+	unless ( $self->{trace} ) {
+	    open (STDERR, "/dev/null" )
+	       or die "can't redirect stderr to /dev/null: $!\n";
+	}
 
-    # This is where we grab the data
-    if ( ref $self->{input} && defined fileno $self->{input} ) {
-        open ( STDIN, "<&" . fileno $self->{input} )
-          or die "error setting up data input: $!\n";
-    } elsif ( $self->{input} ) {
-        open ( STDIN, $self->{input} )
-          or die "error setting up data input: $!\n";
-    } # Defaults to stdin
+	# This is where we graquitb the data
+	if ( ref $self->{input} ) {
+	    open ( STDIN, "<&" . fileno $self->{input} )
+	      or die "error setting up data input: $!\n";
+	} elsif ( $self->{input} ) {
+	    open ( STDIN, $self->{input} )
+	      or die "error setting up data input: $!\n";
+	} # Defaults to stdin
 
-    # This is where the output goes
-    if ( ref $self->{output} && defined fileno $self->{output} ) {
-        open ( STDOUT, ">&" . fileno $self->{output} )
-          or die "can't redirect stdout to proper output fd: $!\n";
-    } elsif ( $self->{output} ) {
-        open ( STDOUT, ">".$self->{output} )
-          or die "can't open $self->{output} for output: $!\n";
-    } # Defaults to stdout
+	# This is where the output goes
+	if ( ref $self->{output} ) {
+	    open ( STDOUT, ">&" . fileno $self->{output} )
+	      or die "can't redirect stdout to proper output fd: $!\n";
+	} elsif ( $self->{output} ) {
+	    open ( STDOUT, ">".$self->{output} )
+	      or die "can't open $self->{output} for output: $!\n";
+	} # Defaults to stdout
 
-    # Close all open file descriptors except STDIN, STDOUT, STDERR
-    # and the status filedescriptor.
-    #
-    # This is needed for the tie interface which opens pipes which
-    # some ends must be closed in the child.
-    #
-    # Besides this is just plain good hygiene
-    my $max_fd = POSIX::sysconf( &POSIX::_SC_OPEN_MAX ) || 256;
-    foreach my $f ( 3 .. $max_fd ) {
-        next if $f == fileno $self->{status_fd};
-        POSIX::close( $f );
-    }
-
-    exec ( @$cmdline )
-      or CORE::die "can't exec gnupg: $!\n";
+	exec ( @$cmdline )
+	  or die "can't exec gnupg: $!\n";
     }
 }
 
 sub cpr_maybe_send($$$) {
-    ($_[0])->cpr_send( @_[1, $#_], 1);
+    (shift)->cpr_send( @_, 1);
 }
 
 sub cpr_send($$$;$) {
     my ($self,$key,$value, $optional) = @_;
 
     my ( $cmd, $arg ) = $self->read_from_status;
-    unless ( defined $cmd && $cmd =~ /^SHM_GET/) {
-    $self->abort_gnupg( "protocol error: expected SHM_GET_XXX got $cmd\n" )
-      unless $optional;
-    $self->next_status( $cmd, $arg );
-    return;
+    unless ( $cmd =~ /^SHM_GET/) {
+	$self->abort_gnupg( "protocol error: expected SHM_GET_XXX got $cmd\n" )
+	  unless $optional;
+	$self->next_status( $cmd, $arg );
+	return;
     }
 
     unless ( $arg eq $key ) {
-    $self->abort_gnupg ( "protocol error: expected key $key got $arg\n" )
-      unless $optional;
-    return;
+	$self->abort_gnupg ( "protocol error: expected key $key got $arg\n" )
+	  unless $optional;
+	return;
     }
 
-    my $shmid        = $self->{shmid};
-    my $shm_size    = $self->{shm_size};
+    my $shmid		= $self->{shmid};
+    my $shm_size	= $self->{shm_size};
 
-    my $offset = 0;
+    my $offset;
     shmread $shmid,$offset,0,2
       or $self->abort_gnupg( "shared memory error: $!\n" );
 
@@ -307,13 +291,13 @@ sub cpr_send($$$;$) {
       if ( ( $shm_size - $offset ) < length $value );
 
     if ( $cmd eq "SHM_GET_BOOL" ) {
-    my $truth = $value ? 1 : 0;
-    shmwrite $shmid, pack( "nC", 1, $truth ), $offset, 3
-      or $self->abort_gnupg( "shared memory error: $!\n" );
+	my $truth = $value ? 1 : 0;
+	shmwrite $shmid, pack( "nC", 1, $truth ), $offset, 3
+	  or $self->abort_gnupg( "shared memory error: $!\n" );
     } else {
-    my $len = length $value;
-    shmwrite $shmid, pack( "na*", $len, $value ), $offset, $len + 2
-      or $self->abort_gnupg( "shared memory error: $!\n" );
+	my $len = length $value;
+	shmwrite $shmid, pack( "na*", $len, $value ), $offset, $len + 2
+	  or $self->abort_gnupg( "shared memory error: $!\n" );
     }
 
     # Set data ready flag
@@ -327,15 +311,13 @@ sub send_passphrase($$) {
 
     # GnuPG should now tell us that it needs a passphrase
     my $cmd = $self->read_from_status;
-    # Skip UserID hint
-    $cmd = $self->read_from_status if ( $cmd =~ /USERID_HINT/ );
     $self->abort_gnupg( "Protocol error: expected NEED_PASSPHRASE.* got $cmd\n")
       unless $cmd =~ /NEED_PASSPHRASE/;
     $self->cpr_send( "passphrase.enter", $passwd );
     unless ( $passwd ) {
-    my $cmd = $self->read_from_status;
-    $self->abort_gnupg( "Protocol error: expected MISSING_PASSPHRASE got $cmd\n" )
-      unless $cmd eq "MISSING_PASSPHRASE";
+	my $cmd = $self->read_from_status;
+	$self->abort_gnupg( "Protocol error: expected MISSING_PASSPHRASE got $cmd\n" )
+	  unless $cmd eq "MISSING_PASSPHRASE";
     }
 }
 
@@ -347,24 +329,24 @@ sub new($%) {
 
     my $self = {};
     if ($args{homedir}) {
-    croak ( "Invalid home directory: $args{homedir}\n")
-      unless -d $args{homedir} && -w _ && -x _;
-    $self->{homedir} = $args{homedir};
+	croak ( "Invalid home directory: $args{homedir}\n")
+	  unless -d $args{homedir} && -w _ && -x _;
+	$self->{homedir} = $args{homedir};
     }
     if ($args{options}) {
-    croak ( "Invalid options file: $args{options}\n")
-      unless -r $args{options};
-    $self->{options} = $args{options};
+	croak ( "Invalid options file: $args{options}\n")
+	  unless -r $args{options};
+	$self->{options} = $args{options};
     }
     if ( $args{gnupg_path} ) {
-    croak ( "Invalid gpg path: $args{gnupg_path}\n")
-      unless -x $args{gnupg_path};
-    $self->{gnupg_path} = $args{gnupg_path};
+	croak ( "Invalid gpg path: $args{gnupg_path}\n")
+	  unless -x $args{gnupath_path};
+	$self->{gnupg_path} = $args{gnupg_path};
     } else {
-    my ($path) = grep { -x "$_/gpg" } split /:/, $ENV{PATH};
-    croak ( "Couldn't find gpg in PATH ($ENV{PATH})\n" )
-      unless $path;
-    $self->{gnupg_path} = "$path/gpg";
+	my ($path) = grep { -x "$_/gpg" } split /:/, $ENV{PATH};
+	croak ( "Couldn't find gpg in PATH" )
+	  unless $path;
+	$self->{gnupg_path} = "$path/gpg";
     }
     $self->{trace} = $args{trace} ? 1 : 0;
 
@@ -375,51 +357,51 @@ sub DESTROY {
     my $self = shift;
     # Signal our child that it is the end
     if ($self->{gnupg_pid} && kill 0 => $self->{gnupg_pid} ) {
-    kill INT => $self->{gnupg_pid};
+	kill INT => $self->{gnupg_pid};
     }
 }
 
 sub gen_key($%) {
     my ($self,%args) = @_;
 
-    my $algo      = $args{algo};
+    my $algo	  = $args{algo};
     $algo ||= DSA_ELGAMAL;
 
-    my $size      = $args{size};
+    my $size	  = $args{size};
     $size ||= 1024;
     croak ( "Keysize is too small: $size" ) if $size < 768;
     croak ( "Keysize is too big: $size" )   if $size > 2048;
 
-    my $expire      = $args{valid};
-    $expire          ||= 0;
+    my $expire	  = $args{valid};
+    $expire		  ||= 0;
 
     my $passphrase = $args{passphrase} || "";
-    my $name      = $args{name};
+    my $name	  = $args{name};
 
-    croak "Missing key name\n"      unless $name;
-    croak "Invalid name: $name\n"
+    croak "Missing key name\n"	  unless $name;
+    croak "Invalid name: $name\n" 
       unless $name =~ /^\s*[^0-9\<\(\[\]\)\>][^\<\(\[\]\)\>]+$/;
 
-    my $email      = $args{email};
+    my $email	  = $args{email};
     if ( $email ) {
-    croak "Invalid email address: $email"
-      unless $email =~ /^\s*        # Whitespace are okay
-                [a-zA-Z0-9_-]    # Doesn't start with a dot
-                [a-zA-Z0-9_.-]*
-                \@        # Contains at most one at
-                [a-zA-Z0-9_.-]+
-                [a-zA-Z0-9_-]    # Doesn't end in a dot
-                   /x
-                 && $email !~ /\.\./;
+	croak "Invalid email address: $email"
+	  unless $email =~ /^\s*		# Whitespace are okay
+				[a-zA-Z0-9_-]	# Doesn't start with a dot
+				[a-zA-Z0-9_.-]*
+				\@		# Contains at most one at
+				[a-zA-Z0-9_.-]+
+				[a-zA-Z0-9_-]	# Doesn't end in a dot
+			       /x 
+				 && $email !~ /\.\./;
     } else {
-    $email = "";
+	$email = "";
     }
 
-    my $comment      = $args{comment};
+    my $comment	  = $args{comment};
     if ( $comment ) {
-    croak "Invalid characters in comment" if $comment =~ /[()]/;
+	croak "Invalid characters in comment" if $comment =~ /[()]/;
     } else {
-    $comment = "";
+	$comment = "";
     }
 
     $self->command( "gen-key" );
@@ -430,15 +412,15 @@ sub gen_key($%) {
 
     $self->cpr_send("keygen.algo", $algo );
     if ( $algo == ELGAMAL ) {
-        # Shitty interactive program, yes I'm sure.
-        # I'm a program, I can't change my mind now.
-        $self->cpr_send( "keygen.algo.elg_se", 1 )
+	# Shitty interactive program, yes I'm sure.
+	# I'm a program, I can't change my mind now.
+	$self->cpr_send( "keygen.algo.elg_se", 1 )
     }
-    $self->cpr_send( "keygen.size",        $size );
-    $self->cpr_send( "keygen.valid",    $expire );
-    $self->cpr_send( "keygen.name",        $name );
-    $self->cpr_send( "keygen.email",    $email );
-    $self->cpr_send( "keygen.comment",    $comment );
+    $self->cpr_send( "keygen.size",		$size );
+    $self->cpr_send( "keygen.valid",	$expire );
+    $self->cpr_send( "keygen.name",		$name );
+    $self->cpr_send( "keygen.email",	$email );
+    $self->cpr_send( "keygen.comment",	$comment );
 
     $self->send_passphrase( $passphrase );
 
@@ -456,29 +438,30 @@ sub import_keys($%) {
 
     my $count;
     if ( ref $args{keys} ) {
-    $self->args( $args{keys} );
+	$self->args( $args{keys} );
     } else {
-    # Only one file to import
-    $self->{input} = $args{keys};
-    $self->args( [] );
+	# Only one file to import
+	$self->{input} = $args{keys};
+	$self->args( [] );
     }
 
     $self->run_gnupg;
   FILE:
     my $num_files = ref $args{keys} ? @{$args{keys}} : 1;
     my ($cmd,$arg);
+    for ( my $i=0; $i < $num_files; ++$i) {
+	# We will see one IMPORTED for each key that is imported
+	KEY:
+	  while ( 1 ) {
+	      ($cmd,$arg) = $self->read_from_status;
+	      last KEY unless $cmd =~ /IMPORTED/;
+	      $count++
+	  }
 
-    # We will see one IMPORTED for each key that is imported
-  KEY:
-    while ( 1 ) {
-    ($cmd,$arg) = $self->read_from_status;
-    last KEY unless $cmd =~ /IMPORTED/;
-    $count++
+	# We will see on IMPORT_RES for all files processed
+	$self->abort_gnupg ( "protocol error expected IMPORT_RES got $cmd\n" )
+	  unless $cmd =~ /IMPORT_RES/;
     }
-
-    # We will see one IMPORT_RES for all files processed
-    $self->abort_gnupg ( "protocol error expected IMPORT_OK got $cmd\n" )
-      unless $cmd =~ /IMPORT_OK/;
     $self->end_gnupg;
 
     # We return the number of imported keys
@@ -489,22 +472,22 @@ sub export_keys($%) {
     my ($self,%args) = @_;
 
     my $options = [];
-    push @$options, "--armor"        if $args{armor};
+    push @$options, "--armor"	    if $args{armor};
 
     $self->{output} = $args{output};
 
     my $keys = [];
     if ( $args{keys}) {
-    push @$keys,
-      ref $args{keys} ? @{$args{keys}} : $args{keys};
+	push @$keys,
+	  ref $args{keys} ? @{$args{keys}} : $args{keys};
     }
 
     if ( $args{secret} ) {
-    $self->command( "export-secret-keys" );
+	$self->command( "export-secret-keys" );
     } elsif ( $args{all} ){
-    $self->command( "export-all" );
+	$self->command( "export-all" );
     } else {
-    $self->command( "export" );
+	$self->command( "export" );
     }
     $self->options( $options );
     $self->args( $keys );
@@ -521,22 +504,21 @@ sub encrypt($%) {
       unless $args{recipient} or $args{symmetric};
     push @$options, "--recipient" => $args{recipient};
 
-    push @$options, "--sign"        if $args{sign};
+    push @$options, "--sign"	    if $args{sign};
     croak ( "can't sign an symmetric encrypted message\n" )
       if $args{sign} and $args{symmetric};
 
     my $passphrase  = $args{passphrase} || "";
 
-    push @$options, "--armor"        if $args{armor};
-    push @$options, "--local-user", $args{"local-user"}
-      if defined $args{"local-user"};
+    push @$options, "--armor"	    if $args{armor};
+    push @$options, "--local-user"  if $args{"local-user"};
 
-    $self->{input}  = $args{plaintext} || $args{input};
+    $self->{input}  = $args{plaintext};
     $self->{output} = $args{output};
     if ( $args{symmetric} ) {
-    $self->command( "symmetric" );
+	$self->command( "symmetric" );
     } else {
-    $self->command( "encrypt" );
+	$self->command( "encrypt" );
     }
     $self->options( $options );
     $self->args( [] );
@@ -545,19 +527,19 @@ sub encrypt($%) {
 
     # Unless we decided to sign or are using symmetric cipher, we are done
     if ( $args{sign} or $args{symmetric} ) {
-        $self->send_passphrase( $passphrase );
-        if ( $args{sign} ) {
-            my ($cmd,$line) = $self->read_from_status;
-            $self->abort_gnupg( "invalid passphrase - $cmd\n" )
-              unless $cmd =~ /GOOD_PASSPHRASE/;
-        }
+	$self->send_passphrase( $passphrase );
+	if ( $args{sign} ) {
+	    my ($cmd,$line) = $self->read_from_status;
+	    $self->abort_gnupg( "invalid passphrase\n" )
+	      unless $cmd =~ /GOOD_PASSPHRASE/;
+	}
     }
 
     # It is possible that this key has no assigned trust value.
     # Assume the caller knows what he is doing.
     $self->cpr_maybe_send( "untrusted_key.override", 1 );
 
-    $self->end_gnupg unless $args{tie_mode};
+    $self->end_gnupg;
 }
 
 sub sign($%) {
@@ -566,18 +548,17 @@ sub sign($%) {
     my $options = [];
     my $passphrase  = $args{passphrase} || "";
 
-    push @$options, "--armor"        if $args{armor};
-    push @$options, "--local-user", $args{"local-user"}
-      if defined $args{"local-user"};
+    push @$options, "--armor"	    if $args{armor};
+    push @$options, "--local-user"   if $args{"local-user"};
 
-    $self->{input}  = $args{plaintext} || $args{input};
+    $self->{input}  = $args{plaintext};
     $self->{output} = $args{output};
     if ( $args{clearsign} ) {
-    $self->command( "clearsign" );
+	$self->command( "clearsign" );
     } elsif ( $args{"detach-sign"}) {
-    $self->command( "detach-sign" );
+	$self->command( "detach-sign" );
     } else {
-    $self->command( "sign" );
+	$self->command( "sign" );
     }
     $self->options( $options );
     $self->args( [] );
@@ -586,11 +567,8 @@ sub sign($%) {
 
     # We need to unlock the private key
     $self->send_passphrase( $passphrase );
-    my ($cmd,$line) = $self->read_from_status;
-    $self->abort_gnupg( "invalid passphrase - $cmd\n" )
-      unless $cmd =~ /GOOD_PASSPHRASE/;
 
-    $self->end_gnupg unless $args{tie_mode};
+    $self->end_gnupg;
 }
 
 sub clearsign($%) {
@@ -606,34 +584,12 @@ sub check_sig($;$$) {
     # signature reporting.
     ($cmd,$arg) = $self->read_from_status unless ( $cmd );
 
-    # Ignore patent warnings.
-    ( $cmd, $arg ) = $self->read_from_status()
-      if ( $cmd =~ /RSA_OR_IDEA/ );
-
-    # Ignore automatic key imports
-    ( $cmd, $arg ) = $self->read_from_status()
-      if ( $cmd =~ /IMPORTED/ );
-
-    ( $cmd, $arg ) = $self->read_from_status()
-      if ( $cmd =~ /IMPORT_OK/ );
-
-    ( $cmd, $arg ) = $self->read_from_status()
-      if ( $cmd =~ /IMPORT_RES/ );
-
     $self->abort_gnupg( "invalid signature from ", $arg =~ /[^ ](.+)/, "\n" )
       if ( $cmd =~ /BADSIG/);
 
-    if ( $cmd =~ /ERRSIG/)
-      {
-        my ($keyid, $key_algo, $digest_algo, $sig_class, $timestamp, $rc)
-           = split ' ', $arg;
-        if ($rc == 9)
-          {
-            ($cmd, $arg) = $self->read_from_status();
-            $self->abort_gnupg( "no public key $keyid" );
-          }
-        $self->abort_gnupg( "error verifying signature from $keyid" )
-      }
+    $self->abort_gnupg( "error verifying signature from ", 
+			$arg =~ /([^ ])/, "\n" )
+      if ( $cmd =~ /ERRSIG/);
 
     $self->abort_gnupg ( "protocol error: expected SIG_ID" )
       unless $cmd =~ /SIG_ID/;
@@ -645,12 +601,6 @@ sub check_sig($;$$) {
     my ( $keyid, $name ) = split /\s+/, $arg, 2;
 
     ( $cmd, $arg ) = $self->read_from_status;
-    my $policy_url = undef;
-    if ( $cmd =~ /POLICY_URL/ ) {
-        $policy_url = $arg;
-        ( $cmd, $arg ) = $self->read_from_status;
-    }
-
     $self->abort_gnupg ( "protocol error: expected VALIDSIG" )
       unless $cmd =~ /VALIDSIG/;
     my ( $fingerprint ) = split /\s+/, $arg, 2;
@@ -660,15 +610,14 @@ sub check_sig($;$$) {
       unless $cmd =~ /TRUST/;
     my ($trust) = parse_trust( $cmd );
 
-    return { sigid        => $sigid,
-         date        => $date,
-         timestamp        => $time,
-         keyid        => $keyid,
-         user        => $name,
-         fingerprint    => $fingerprint,
-         trust        => $trust,
-         policy_url        => $policy_url,
-       };
+    return { sigid	    => $sigid,
+	     date	    => $date,
+	     timestamp	    => $time,
+	     keyid	    => $keyid,
+	     user	    => $name,
+	     fingerprint    => $fingerprint,
+	     trust	    => $trust,
+	   };
 }
 
 sub verify($%) {
@@ -677,12 +626,12 @@ sub verify($%) {
     croak ( "missing signature argument\n" ) unless $args{signature};
     my $files = [];
     if ( $args{file} ) {
-    croak ( "detached signature must be in a file\n" )
-      unless -f $args{signature};
-    push @$files, $args{signature},
-      ref $args{file} ? @{$args{file}} : $args{file};
+	croak ( "detached signature must be in a file\n" ) 
+	  unless -f $args{signature};
+	push @$files, $args{signature}, 
+	  ref $args{file} ? @{$args{file}} : $args{file};
     } else {
-    $self->{input} = $args{signature};
+	$self->{input} = $args{signature};
     }
     $self->command( "verify" );
     $self->options( [] );
@@ -697,10 +646,11 @@ sub verify($%) {
 }
 
 sub decrypt($%) {
-    my $self = shift;
-    my %args = @_;
+    my ($self,%args) = @_;
 
-    $self->{input}  = $args{ciphertext} || $args{input};
+    my $passphrase  = $args{passphrase} || "";
+
+    $self->{input}  = $args{ciphertext};
     $self->{output} = $args{output};
     $self->command( "decrypt" );
     $self->options( [] );
@@ -708,69 +658,36 @@ sub decrypt($%) {
 
     $self->run_gnupg;
 
-    $self->decrypt_postwrite( @_ ) unless $args{tie_mode};
-}
-
-sub decrypt_postwrite($%) {
-    my ($self,%args) = @_;
-
-    my $passphrase  = $args{passphrase} || "";
-
     my ( $cmd, $arg );
     unless ( $args{symmetric} ) {
-    ( $cmd, $arg ) = $self->read_from_status;
-    $self->abort_gnupg ( "protocol error: expected ENC_TO got $cmd: \n" )
-      unless $cmd =~ /ENC_TO/;
+	( $cmd, $arg ) = $self->read_from_status;
+	$self->abort_gnupg ( "protocol error: expected ENC_TO got $cmd: \n" )
+	  unless $cmd =~ /ENC_TO/;
     }
 
     $self->send_passphrase( $passphrase );
 
     ($cmd,$arg) = $self->read_from_status;
-    $self->abort_gnupg ( "invalid passphrase - $cmd\n" )
+    $self->abort_gnupg ( "invalid passphrase\n" )
       if $cmd =~ /BAD_PASSPHRASE/;
     my $sig = undef;
-    if ( ! $args{symmetric} ) {
-    $self->abort_gnupg ( "protocol error: expected GOOD_PASSPHRASE got $cmd: \n" )
-      unless $cmd =~ /GOOD_PASSPHRASE/;
+    if ( not $args{symmetric} ) {
+	$self->abort_gnupg ( "protocol error: expected GOOD_PASSPHRASE got $cmd: \n" )
+	  unless $cmd =~ /GOOD_PASSPHRASE/;
+	($cmd,$arg) = $self->read_from_status;
 
-    $sig = $self->decrypt_postread() unless $args{tie_mode};
+	if ( $cmd =~ /SIG_ID/ ) {
+	    $sig = $self->check_sig( $cmd, $arg );
+	    ( $cmd, $arg ) = $self->read_from_status;
+	}
+	$self->abort_gnupg( "protocol error: expected DECRYPTION_OKAY got $cmd: \n" )
+	  unless $cmd =~ /DECRYPTION_OKAY/;
     } else {
-        # gnupg 1.0.2 adds this status message
-        ( $cmd, $arg ) = $self->read_from_status() if $cmd =~ /BEGIN_DECRYPTION/;
-
-        $self->abort_gnupg( "invalid passphrase - $cmd" ) unless $cmd =~ /PLAINTEXT/;
+	$self->abort_gnupg( "invalid passphrase" )
+	  unless $cmd =~ /DECRYPTION_OKAY/;
     }
 
-    $self->end_gnupg() unless $args{tie_mode};
-
-    return $sig ? $sig : 1;
-}
-
-sub decrypt_postread($) {
-    my $self = shift;
-
-    my @cmds;
-    # gnupg 1.0.2 adds this status message
-    my ( $cmd, $arg ) = $self->read_from_status;
-    push @cmds, $cmd;
-
-    if ($cmd =~ /BEGIN_DECRYPTION/) {
-    ( $cmd, $arg ) = $self->read_from_status();
-    push @cmds, $cmd;
-    };
-
-    my $sig = undef;
-    while (defined $cmd && !($cmd =~ /DECRYPTION_OKAY/)) {
-    if ( $cmd =~ /SIG_ID/ ) {
-        $sig = $self->check_sig( $cmd, $arg );
-    }
-    ( $cmd, $arg ) = $self->read_from_status();
-    push @cmds, $cmd if defined $cmd;
-    };
-
-    my $cmds = join ', ', @cmds;
-    $self->abort_gnupg( "protocol error: expected DECRYPTION_OKAY but never got it (all I saw was: $cmds): \n" )
-      unless $cmd =~ /DECRYPTION_OKAY/;
+    $self->end_gnupg;
 
     return $sig ? $sig : 1;
 }
@@ -790,26 +707,26 @@ GnuPG - Perl module interface to the GNU Privacy Guard.
 
     my $gpg = new GnuPG();
 
-    $gpg->encrypt(  plaintext    => "file.txt",    output        => "file.gpg",
-            armor    => 1,         sign    => 1,
-            passphrase  => $secret );
+    $gpg->encrypt(  plaintext	=> "file.txt",	output	    => "file.gpg",
+		    armor	=> 1,		 sign	=> 1,
+		    passphrase  => $secret );
 
-    $gpg->decrypt( ciphertext    => "file.gpg",    output        => "file.txt" );
+    $gpg->decrypt( ciphertext	=> "file.gpg",	output	    => "file.txt" );
 
     $gpg->clearsign( plaintext => "file.txt", output => "file.txt.asc",
-             passphrase => $secret,   armor => 1,
-            );
+		     passphrase => $secret,   armor => 1,
+		    );
 
     $gpg->verify( signature => "file.txt.asc", file => "file.txt" );
 
-    $gpg->gen_key( name => "Joe Blow",        comment => "My GnuPG key",
-           passphrase => $secret,
-            );
+    $gpg->gen_key( name => "Joe Blow",	    comment => "My GnuPG key",
+		   passphrase => $secret,
+		    );
 
 =head1 DESCRIPTION
 
 GnuPG is a perl interface to the GNU Privacy Guard. It uses the
-shared memory coprocess interface that gpg provides for its
+shared memory coprocess interface that gpg provides for its 
 wrappers. It tries its best to map the interactive interface of
 the gpg to a more programmatic model.
 
@@ -821,14 +738,11 @@ argument using named parameters, and errors are returned by
 throwing an exception (using croak).  If you wan't to catch
 errors you will have to use eval.
 
-There is also a tied file handle interface which you may find more
-convenient for encryption and decryption. See GnuPG::Tie(3) for details.
-
 =head1 CONSTRUCTOR
 
 =head2 new ( [params] )
 
-You create a new GnuPG wrapper object by invoking its new method.
+You create a new GnuPG wrapper object by invoking its new method.  
 (How original !).  The module will try to finds the B<gpg> program
 in your path and will croak if it can't find it. Here are the
 parameters that it accepts :
@@ -869,7 +783,6 @@ keyboard while running this methods because it consumes a lot of
 entropy from the computer. Here are the parameters it accepts :
 
     Ex: $gpg->
-
 =over
 
 =item algo
@@ -890,7 +803,7 @@ How long the key is valid. Defaults to 0 or never expire.
 
 =item name
 
-This is the only mandatory argument. This is the name that will used
+This is the only mandatory argument. This is the name that will used 
 to construct the user id.
 
 =item email
@@ -905,11 +818,11 @@ Optional comment portion of the user id.
 
 The passphrase that will be used to encrypt the private key. Optional
 but strongly recommended.
-
+ 
 =back
 
     Example: $gpg->gen_key( algo => DSA_ELGAMAL, size => 1024,
-                name => "My name" );
+			    name => "My name" );
 
 =head2 import_keys( [params] )
 
@@ -938,7 +851,7 @@ encounters an error. Parameters :
 
 =item keys
 
-Optional argument that restricts the keys that will be exported.
+Optional argument that restricts the keys that will be exported. 
 Can either be a user id or a reference to an array of userid that
 specifies the keys to be exported. If left unspecified, all keys
 will be exported.
@@ -956,7 +869,7 @@ OpenPGP compliant) will be exported.
 =item output
 
 This argument specifies where the keys will be exported. Can be either
-a file name or a reference to a file handle. If not specified, the
+a file name or a reference to a file handle. If not specified, the 
 keys will be exported to stdout.
 
 =item armor
@@ -1029,8 +942,8 @@ key.
 =back
 
     Example: $gpg->encrypt( plaintext => file.txt, output => "file.gpg",
-                sign => 1, passphrase => $secret
-                );
+			    sign => 1, passphrase => $secret
+			    );
 
 =head2 sign( [params] )
 
@@ -1074,8 +987,8 @@ the whole file.
 =back
 
     Example: $gpg->sign( plaintext => "file.txt", output => "file.txt.asc",
-             armor => 1,
-             );
+			 armor => 1,
+			 );
 
 =head2 clearsign( [params] )
 
@@ -1140,13 +1053,13 @@ The fingerprint of the signature.
 =item trust
 
 The trust value of the public key of the signer. Those are values that
-can be imported in your namespace with the :trust tag. They are
+can be imported in your namespace with the :trust tag. They are 
 (TRUST_UNDEFINED, TRUST_NEVER, TRUST_MARGINAL, TRUST_FULLY, TRUST_ULTIMATE).
 
 =back
 
     Example : my $sig = $gpg->verify( signature => "file.txt.asc",
-                      file => "file.txt" );
+				      file => "file.txt" );
 
 =head2 decrypt( [params] )
 
@@ -1160,7 +1073,7 @@ true. Method parameters :
 
 =item ciphertext
 
-This optional parameter contains either the name of the file
+This optional parameter contains either the name of the file 
 containing the ciphertext or a reference to a file handle containing
 the ciphertext. If not present, STDIN will be decrypted.
 
@@ -1184,20 +1097,16 @@ message.
 
 =back
 
-    Example: $gpg->decrypt( ciphertext => "file.gpg", output => "file.txt"
-                passphrase => $secret );
+    Example: $gpg->decrypt( ciphertext => "file.gpg", output => "file.txt" 
+			    passphrase => $secret );
 
 =head1 AUTHOR
 
-Francis J. Lacoste <francis.lacoste@Contre.COM>
+Francis J. Lacoste <francis.lacoste@iNsu.COM>
 
 =head1 COPYRIGHT
 
-Copyright (c) 1999,2000 iNsu Innovations. Inc.
-Copyright (c) 2001 Francis J. Lacoste
-
-:wq!
-
+Copyright (c) 1999 Francis J. Lacoste and iNsu Innovations. inc.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -1206,6 +1115,6 @@ the Free Software Foundation; either version 2 of the License, or
 
 =head1 SEE ALSO
 
-gpg(1) GnuPG::Tie(3)
+gpg(1) gpgmailtunl(1)
 
 =cut
